// MakeMispGetFeature.java
// hiroki.matsui@aist.go.jp

import java.io.*;
import java.util.regex.*;

public class MakeMispGetFeature{
  static private final String HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<misp:GetFeature\n  xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\"\n  xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n  xmlns:misp=\"http://www.infosharp.org/misp\"\n  xmlns:gml=\"http://www.opengis.net/gml\"\n  >";
  static private final String QUERY_PREFIX = "<misp:Query typeName=\"";
  static private final String QUERY_MIDFIX = "\"\n  xmlns=\"";
  static private final String QUERY_SUFFIX = "\"\n>";
  static private final String FILTER = "<misp:Filter>\n  <misp:True/>\n</misp:Filter>";
  static private final String FOOTER = "</misp:Query>\n</misp:GetFeature>";

  static private final Pattern NAMESPACE_PATTERN = Pattern.compile("targetNamespace=\"(.+?)\"");
  static private final Pattern FEATURE_NAME_PATTERN = Pattern.compile("xsd:element name=\"(.+?)\"");

  static private String featureTypeName;
  static private String namespace;

  public static void main(String[] args) throws IOException{
    if(args.length == 0 || (! args[0].equals("-n") && ! args[0].equals("-s"))){
      printUsage();
      System.exit(0);
    }

    if(args[0].equals("-n")){
      // name and targetNamespace mode
      if(args.length != 3){
	printUsageForWinCmd(args[0]);
	System.exit(0);
      }else{
	featureTypeName = args[1];
	namespace = args[2];
      }
    }else if(args[0].equals("-s")){
      // schema file mode
      if(args.length != 2){
	printUsageForWinCmd(args[0]);
	System.exit(0);
      }else{
	setFeatureTypeWithSchema(args[1]);
      }
    }


    System.out.println(makeMispGetFeature());
  }


  private static void printUsage(){
    System.err.println("Usage: java MakeMispGetFeature");
    System.err.println("         -n FeatureTypeName Namespace");
    System.err.println("         -s SchemaFile.xsd");
  }

  private static void printUsageForWinCmd(String option){
    if(option.equals("-n")){
      System.err.println("Usage: makeMispGetFeatureAll FeatureTypeName Namespace > out.misp.GetFeature.xml");
      System.err.println("  ex. makeMispGetFeatureAll DataTypeName http://www.example.com/base > dataType.misp.GetFeature.xml");
    }else if(option.equals("-s")){
      System.err.println("Usage: makeMispGetFeatureAllWithSchema SchemaFile.xsd");
      System.err.println("  ex. makeMispGetFeatureAllWithSchema dataType.xsd");
    }
  }

  private static String makeMispGetFeature(){
    return (HEADER + "\n" + 
	    QUERY_PREFIX + featureTypeName + 
	    QUERY_MIDFIX + namespace + QUERY_SUFFIX + "\n" +
	    FILTER + "\n" + FOOTER
	    );
  }

  private static void setFeatureTypeWithSchema(String schemaFile) throws IOException{
    try{
      String line;
      BufferedReader reader = new BufferedReader(new FileReader(schemaFile));

      while((line = reader.readLine()) != null){
	Matcher m = NAMESPACE_PATTERN.matcher(line);
	if(m.find()){
	  namespace = m.group(1);
	  break;
	}
      }
      while((line = reader.readLine()) != null){
	Matcher m = FEATURE_NAME_PATTERN.matcher(line);
	if(m.find()){
	  featureTypeName = m.group(1);
	  break;
	}
      }

      reader.close();
    }catch(FileNotFoundException fnfe){
      System.err.println("ꤵ줿ޥե뤬Ĥޤ");
      System.err.println(schemaFile);
      System.exit(0);
    }
  }
}
